/*! \file debug.c

  \brief debug utility.

  \author
    Copyright (C) SEIKO EPSON CORPORATION 2010.

    Driver for Epson SIDM Printers

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/stat.h>
#include <stdarg.h>
#include <libgen.h>
#include <stdlib.h>             /* extern char **environ; */
#include "debug.h"

#if (0 != MYDEBUG)

#define DEBUG_LOG_FILE_PATH        "/tmp"      /*!< \brief debug log   file directory */
#define DEBUG_LOG_FILE_SUFFIX      "debug.log" /*!< \brief debug log   file suffix */
#define DEBUG_IMAGEOUT_FILE_PATH   "/tmp"      /*!< \brief debug image file directory */
#define DEBUG_IMAGEOUT_FILE_SUFFIX "debug.raw" /*!< \brief debug image file suffix */

extern char **environ;          /*!< \brief environment parameter list */

/*
 * File scope global parameters...
 */

static FILE *Debug_log_fp = NULL;      /*!< \brief File stream pointer for debug log file. */
static FILE *Debug_imageout_fp = NULL; /*!< \brief File stream pointer for CUPS filter input raster image data file. */





/*! \brief open output files for debug.

    Open a debug log file.
    Open a debug image data file for CUPS filter input raster image.

    \li debug log  file path:  #DEBUG_LOG_FILE_PATH / \a printer_name - \a job_id - #DEBUG_LOG_FILE_SUFFIX
    \li image data file path:  #DEBUG_IMAGEOUT_FILE_PATH / \a printer_name - \a job_id - #DEBUG_IMAGEOUT_FILE_SUFFIX

    \param  printer_name    A printer name by the CUPS.
    \param  job_id          A job ID by the CUPS.

    \see #debug_close(), #debug_log(), #debug_image_out(), #debug_check_memory_allocate_counter()
 */
FUNCTION void debug_open( char *printer_name, const char *job_id )
{
  char  filename[MAX_STR_BUF];
  int   status;
  char *name;
  int   number;

  if ( ARGERROR( NULL ==  printer_name ) ) { return; }
  if ( ARGERROR( NULL ==  job_id       ) ) { return; }
  if ( ARGERROR(    0 == *printer_name ) ) { return; }
  if ( ARGERROR(    0 == *job_id       ) ) { return; }

  name = basename(printer_name);
  number = atoi(job_id);

  ASSERT(MAX_STR_BUF > (strlen(DEBUG_LOG_FILE_PATH) + strlen(name) + strlen(job_id) + strlen(DEBUG_LOG_FILE_SUFFIX) + 3));

  if ( (NULL != Debug_log_fp) || (NULL == Debug_imageout_fp) ) {
    debug_close();
  }

  status = snprintf(filename, MAX_STR_BUF, "%s/%s-%04d-%s", DEBUG_LOG_FILE_PATH, name, number, DEBUG_LOG_FILE_SUFFIX);
  if ( 0 < status ) {
    Debug_log_fp = fopen(filename, "w");
    chmod(filename, 0666);
  } else {
    return;
  }

  status = snprintf(filename, MAX_STR_BUF, "%s/%s-%04d-%s", DEBUG_IMAGEOUT_FILE_PATH, name, number, DEBUG_IMAGEOUT_FILE_SUFFIX);
  if ( 0 < status ) {
    Debug_imageout_fp = fopen(filename, "w");
    chmod(filename, 0666);
  } else {
    return;
  }

  return;
}


/*! \brief close output files for debug.

    Close a debug log file.
    Close a debug image data file for CUPS filter input raster image.

    \see #debug_open()
 */
FUNCTION void debug_close( void )
{
  if ( NULL != Debug_imageout_fp ) {
    fclose(Debug_imageout_fp);
    Debug_imageout_fp = NULL;
  }

  if ( NULL != Debug_log_fp ) {
    fclose(Debug_log_fp);
    Debug_log_fp = NULL;
  }

  return;
}


/*! \brief formated text message output to debug log file.

    Debug information output to debug.log file.
    Like a \c printf(3).

    \param  fmt    Formated message string, like a \c printf(3).
    \param  ...    Argument params for \a fmt, like a \c printf(3).

    \see #debug_open(), #debug_close(), #debug_log()
 */
FUNCTION void dbgprintf(const char *fmt, ... )
{
  va_list ap;
  int n = 0;

  if (         ( NULL == Debug_log_fp ) ) { return; }
  if ( ARGERROR( NULL == fmt          ) ) { return; }

  va_start(ap, fmt);
  n = vfprintf(Debug_log_fp, fmt, ap);
  va_end(ap);

  return;
}



/*! \brief formated text message output to debug log file.

    Debug information output to debug.log file with the location information.

    \param  file   filename
    \param  line   line number in \a file
    \param  func   function name in \a file
    \param  fmt    Formated message string, like a \c printf(3).
    \param  ...    Argument params for \a fmt, like a \c printf(3).

    \see #debug_open(), #debug_close(), #dbgprintf()
 */
FUNCTION void debug_log(const char *file, const int line, const char *func, const char *fmt, ... )
{
  va_list ap;
  int n = 0;

  if (         ( NULL == Debug_log_fp ) ) { return; }
  if ( ARGERROR( NULL == fmt          ) ) { return; }

  fprintf(Debug_log_fp, "%s:%d:%s: ", file, line, func);
  va_start(ap, fmt);
  n = vfprintf(Debug_log_fp, fmt, ap);
  va_end(ap);

  return;
}


/*! \brief binary data output to debug data file..

    Binary data output to file.

    \param  data binary data.
    \param  size output binary data size in Bytes.

    \see #debug_open(), #debug_close()
 */
FUNCTION void debug_image_out( unsigned char *data, size_t size )
{
  size_t writed = 0;
  size_t n = 0;
  int i = 0;

  if (         ( NULL == Debug_imageout_fp ) ) { return; }
  if ( ARGERROR( NULL == data              ) ) { return; }
  if ( ARGERROR(    0 == size              ) ) { return; }

  for ( i = 0; (3 > i) && (n < size); i++ ) {   /* retry limit = 3 */
    writed = fwrite(&data[n], sizeof(unsigned char), (size - n), Debug_imageout_fp);
    n += writed;
  }

  return;
}


/*! \brief Memory alloc/free count check

    Check memory leak.

    \see #debug_open()
 */

FUNCTION void debug_check_memory_allocate_counter( void )
{
  int alloc_counter;
  int free_counter;

  get_my_alloc_counter(&alloc_counter, &free_counter);

  if ( alloc_counter != free_counter ) {
    dbgprintf("!error: alloc_counter != free_counter\n");
    dbgprintf("    alloc_counter = %d\n", alloc_counter);
    dbgprintf("     free_counter = %d\n",  free_counter);
  }
}


/*! \brief show environment parameter

    System environment parameter output to debug log file.

    \see #debug_open()
 */
FUNCTION void debug_show_env( void )
{
  int i;

  dbgprintf("\n* ENV START ************************************************\n");

  for ( i = 0; NULL != environ[i]; i++ ) {
    dbgprintf("%s\n", environ[i]);
  }
  dbgprintf("* ENV END   ************************************************\n");
}



/*! \brief show command line option for CUPS

    Command line option (for CUPS) output to debug log file.

    \param num_cmd_options  number of command-line option for CUPS
    \param cmd_options      command-line option list for CUPS

    \see #debug_open()
 */
FUNCTION void debug_show_cups_cmd_option_list( int num_cmd_options, cups_option_t *cmd_options )
{
  int i;

  dbgprintf("\n* CUPS command-line option list START **********************\n");

  dbgprintf("num_cmd_options = %d\n", num_cmd_options);
  for ( i = 0; i < num_cmd_options; i++ ) {
    dbgprintf("%s=%s\n", cmd_options[i].name, cmd_options[i].value);
  }
  dbgprintf("* CUPS command-line option list END ************************\n");
}


/*! \brief show CUPS page size parameters

    CUPS page size parameters output to debug log file.

    \param page_size  CUPS page size information

    \see #debug_open()
 */
FUNCTION void debug_show_cups_page_size( const ppd_size_t *page_size )
{
  dbgprintf("\n* CUPS page size information (72 dpi) START ****************\n");

  dbgprintf("width  = %f\n", page_size->width);
  dbgprintf("length = %f\n", page_size->length);
  dbgprintf("top    = %f\n", page_size->top);
  dbgprintf("bottom = %f\n", page_size->bottom);
  dbgprintf("left   = %f\n", page_size->left);
  dbgprintf("right  = %f\n", page_size->right);

  dbgprintf("* CUPS page size END ***************************************\n");
}


/*! \brief show ESC/P page size parameters

    ESC/P page size information output to debug log file.

    \param escp_page_info       ESC/P spec page size information
    \param design_base_reso     design base resolution in this driver

    \see #debug_open()
 */
FUNCTION void debug_show_escp_page_size( escp_page_info_t *escp_page_info, unsigned long design_base_reso)
{
  dbgprintf("\n* Page size information in ESC/P (%u dpi) START **********\n", design_base_reso);

  dbgprintf("page_length   = %f\n", escp_page_info->page_length);
  dbgprintf("margin_top    = %f\n", escp_page_info->margin_top);
  dbgprintf("margin_bottom = %f\n", escp_page_info->margin_bottom);
  dbgprintf("margin_left   = %f\n", escp_page_info->margin_left);
  dbgprintf("margin_right  = %f\n", escp_page_info->margin_right);

  dbgprintf("* Page size information END ********************************\n");
}

/*! \brief show raster data information

    CUPS raster data parameters output to debug log file.

    \param  page       page number
    \param  width      cupsWidth
    \param  height     cupsHeight
    \param  row_count  cupsRowCount

    \see #debug_open()
 */
FUNCTION void debug_show_raster_info( int page, unsigned width, unsigned height, unsigned row_count )
{
  dbgprintf("\n* Input CUPS raster data information (pixel) START *********\n");

  dbgprintf("Page=%d\n", page);
  dbgprintf("page width=%d pixel\n", width);
  dbgprintf("page height=%d lines\n", height);
  dbgprintf("band height=%d lines\n", row_count);

  dbgprintf("* Input CUPS raster data information END *******************\n");
}


/*! \brief show raster image data processing information

    raster image data processing (in a page) information output to debug log file.

    \param  scan_line      number of lines in a page
    \param  band_line      number of rasters in the raster image band buffer
    \param  skip_line_num  number of skip line

    \see #debug_open() #rastertoepsidm()
 */
FUNCTION void debug_show_scanline_info( int scan_line, unsigned short band_line, unsigned short skip_line_num )
{
  dbgprintf("  scan_line=%d", scan_line);
  dbgprintf(", band_line=%d", band_line);
  dbgprintf(", skip_line_num=%d", skip_line_num);
  dbgprintf("\n");
}


#endif  /* (0 != MYDEBUG) */

/* end of file */
